/********************************************************************/
/*                                                                  */
/*   MQCAPONE has 2 parameters                                      */
/*                                                                  */
/*      - name of the parameters file                               */
/*      - name of the file to store the captured messages           */
/*                                                                  */
/*    if no queue manager is specified, the default queue manager   */
/*    is used.                                                      */
/*                                                                  */
/********************************************************************/

/********************************************************************/
/*                                                                  */
/* Changes in V1.1                                                  */
/*                                                                  */
/* 1) Added support for Sun Solaris.                                */
/* 2) Use keyword rather than positional arguments.                 */
/*                                                                  */
/********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef SOLARIS
#include <ctype.h>
#include <unistd.h>
#endif

/* includes for MQI */
#include <cmqc.h>

#define MAX_MESSAGE_LENGTH	256 * 65536 + 1

#define QNAME				"QNAME"
#define QMGR				"QMGR"
#define DELIMITER			"DELIMITER"
#define DELIMITERX			"DELIMITERX"
#define STRIPRFH			"STRIPRFH"
#define MAXMSGCOUNT			"MSGCOUNT"
#define RFHSTRIP			1
#define RFHNO				0

static char copyright[] = "(C) Copyright IBM Corp, 2001/2002";
static char Version[]=\
"@(#)MQCapone V1.1 - MQSI V2.0 MQ data capture tool  - Jim MacNair ";

#ifdef _DEBUG
static char Level[]="mqcapone.c V1.1 Debug version ("__DATE__" "__TIME__")";
#else
static char Level[]="mqcapone.c V1.1 Release version ("__DATE__" "__TIME__")";
#endif

static  const unsigned char eatab[] = {  /***** EBCDIC -----> ASCII *****/
                    0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,   /* 00 */
                    0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,0x0F,   /* 08 */
                    0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,   /* 10 */
                    0x18,0x19,0x1A,0x1B,0x1C,0x1D,0x1E,0x1F,   /* 18 */
                    0x20,0x21,0x22,0x23,0x24,0x0A,0x26,0x27,   /* 20 */
                    0x28,0x29,0x2A,0x2B,0x2C,0x2D,0x2E,0x2F,   /* 28 */
                    0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,   /* 30 */
                    0x38,0x39,0x3A,0x3B,0x3C,0x3D,0x3E,0x3F,   /* 38 */
                    0x20,0xFF,0x83,0x84,0x85,0xA0,0xC6,0x86,   /* 40 */
                    0x87,0xA4,0x5B,0x2E,0x3C,0x28,0x2B,0x21,   /* 48 */
                    0x26,0x82,0x88,0x89,0x8A,0xA1,0x8C,0x8B,   /* 50 */
                    0x8D,0xE1,0x5D,0x24,0x2A,0x29,0x3B,0x5E,   /* 58 */
                    0x2D,0x2F,0xB6,0x8E,0xB7,0xB5,0xC7,0x8F,   /* 60 */
                    0x80,0xA5,0xDD,0x2C,0x25,0x5F,0x3E,0x3F,   /* 68 */
                    0x9B,0x90,0xD2,0xD3,0xD4,0xD6,0xD7,0xD8,   /* 70 */
                    0xDE,0x60,0x3A,0x23,0x40,0x27,0x3D,0x22,   /* 78 */
                    0x9D,0x61,0x62,0x63,0x64,0x65,0x66,0x67,   /* 80 */
                    0x68,0x69,0xAE,0xAF,0xD0,0xEC,0xE8,0xF1,   /* 88 */
                    0xF8,0x6A,0x6B,0x6C,0x6D,0x6E,0x6F,0x70,   /* 90 */
                    0x71,0x72,0xA6,0xA7,0x91,0xF7,0x92,0xCF,   /* 98 */
                    0xE6,0x7E,0x73,0x74,0x75,0x76,0x77,0x78,   /* A0 */
                    0x79,0x7A,0xAD,0xA8,0xD1,0xED,0xE7,0xA9,   /* A8 */
                    0xBD,0x9C,0xBE,0xFA,0x9F,0xF5,0xF4,0xAC,   /* B0 */
                    0xAB,0xF3,0xAA,0x7C,0xEE,0xF9,0xEF,0xF2,   /* B8 */
                    0x7B,0x41,0x42,0x43,0x44,0x45,0x46,0x47,   /* C0 */
                    0x48,0x49,0xF0,0x93,0x94,0x95,0xA2,0xE4,   /* C8 */
                    0x7D,0x4A,0x4B,0x4C,0x4D,0x4E,0x4F,0x50,   /* D0 */
                    0x51,0x52,0xD5,0x96,0x81,0x97,0xA3,0x98,   /* D8 */
                    0x5C,0xE1,0x53,0x54,0x55,0x56,0x57,0x58,   /* E0 */
                    0x59,0x5A,0xFD,0xE2,0x99,0xE3,0xE0,0xE5,   /* E8 */
                    0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,   /* F0 */
                    0x38,0x39,0xFC,0xEA,0x9A,0xEB,0xE9,0xFF};  /* F8 */

	/* global error switch */
	int		err=0;

	/* Universal parameters */

	char	saveQname[MQ_Q_NAME_LENGTH + 1];
	char	saveQMname[MQ_Q_MGR_NAME_LENGTH + 1];

	/* name of the parameters file */
	char	parmFilename[512];
	char	outputFilename[512];

/**************************************************************/
/*                                                            */
/* Parameter values from parameter file.                      */
/*                                                            */
/**************************************************************/

	int		striprfh;

	/* Queue and qmgr names to read from */
	char	qname[MQ_Q_NAME_LENGTH + 1];
	char	qmname[MQ_Q_MGR_NAME_LENGTH + 1];

/**************************************************************/
/*                                                            */
/* This routine skips blanks in a string                      */
/*                                                            */
/**************************************************************/

char * skipBlanks(char *str)

{
   while (*str == ' ')
   {
      str++;
   } /* endwhile */

   return str;
}

/**************************************************************/
/*                                                            */
/* This routine finds the next blank in a string              */
/*                                                            */
/**************************************************************/

char * findBlank(char *str)

{
   while ((str[0] != ' ') && (str[0] != 0))
   {
      str++;
   } /* endwhile */

   return str;
}

/**************************************************************/
/*                                                            */
/* This routine translates a string to upper case.            */
/*                                                            */
/**************************************************************/

void strupper(char *str)

{
	int	i=0;
	int	len=strlen(str);

	while (i < len)
	{
		str[i] = toupper(str[i]);
		i++;
	}
}

/**************************************************************/
/*                                                            */
/* Routine to reverse the bytes in an int variable            */
/*                                                            */
/**************************************************************/

int reverseBytes4(int var)

{
    return ((unsigned int) var >> 24) + (((unsigned int) var & 0x00ff0000) >> 8) +
           (((unsigned int) var & 0x0000ff00) << 8) + (((unsigned int) var & 255) <<24);
}

/**************************************************************/
/*                                                            */
/* Translate from EBCDIC to ASCII                             */
/*                                                            */
/**************************************************************/

void EbcdicToAscii(const unsigned char *dati, unsigned int pl, unsigned char *dato)

{
	unsigned int i;
	for(i=0;i<pl;i++)
	{
		dato[i]=eatab[dati[i]];
	}
}

/**************************************************************/
/*                                                            */
/* Return the value of an individual                          */
/* hexadecimal character                                      */
/* the returned value should be                               */
/* between 0 and 15 inclusive.                                */
/*                                                            */
/**************************************************************/

char getHexCharValue(unsigned char charIn)

{
	char	ch;
	char	result=0;

	if ((charIn > '0') && (charIn <= '9'))
	{
		result = charIn & 15;
	}
	else
	{
		ch = toupper(charIn);
		if ((ch >= 'A') && (ch <= 'F'))
		{
			result = ch - 'A' + 10;
		}
	}

	return result;
}

/**************************************************************/
/*                                                            */
/* Translate from Hex to ASCII                                */
/*                                                            */
/**************************************************************/

void HexToAscii(char *dati, int pl, char *dato)

{
	int i;
	int buffer;
	char	ch;

	buffer = 0;
	i = 0;
	while (i < pl)
	{
		ch = getHexCharValue((unsigned char) dati[i++]) << 4;
		ch += getHexCharValue((unsigned char) dati[i++]);
		dato[buffer++] = ch;
	}
}

/**************************************************************/
/*                                                            */
/* Remove the leading and trailing quotes if present.         */
/*                                                            */
/**************************************************************/

char * removeQuotes(char *dati)

{
	char	ch;

	/* check for a leading quote */
	if (('"' == dati[0]) || ((char)148) == dati[0])
	{
		ch = dati[0];

		/* found leading quote, skip it */
		dati++;

		/* make sure the last character is a matching quote */
		if (dati[strlen(dati) - 1] == ch)
		{
			dati[strlen(dati) - 1] = 0;
		}
		else
		{
			printf("***** missing trailing quote for value %s\n", dati);
		}
	}

	return dati;
}

void checkerror(const char *mqcalltype, MQLONG compcode, MQLONG reason, char *resource)

{
	if ((compcode != MQCC_OK) || (reason != MQRC_NONE))
	{
		printf("MQSeries error with %s on %s - compcode = %d, reason = %d\n",
				mqcalltype, resource, compcode, reason);
	}
}

void captureString(char * value, char * arg, const int maxlen)

{
	if ((int) strlen(arg) < maxlen)
	{
		strcpy(value, arg);
	}
	else
	{
		/* issue an error message */
		printf("*****Value of constant exceeds maximum length\n");

		/* Set the return code based on the message number */
		err = 94;
	}
}

int getArgPointer(int argc,
				  char *argv[],
				  int index,
				  char ** parmData,
				  const char * parmDesc)

{
	if (strlen(argv[index]) > 2)
	{
		/* point to the argument data */
		(*parmData) = argv[index] + 2;
	}
	else
	{
		index++;

		if (index < argc)
		{
			/* point to the argument data */
			(*parmData) = argv[index];
		}
		else
		{
			/* issue an error message */
			printf("*****%s parameter is missing value\n", parmDesc);
			
			/* Set the return code to indicate the error */
			err = 95;

			/*point to something valid */
			(*parmData) = argv[index - 1] + 2;
		}
	}


	return index;
}

int processIndArg(int argc, 
				  char *argv[], 
				  int index, 
				  char * parm, 
				  int parmsize, 
				  const char * parmDesc)

{
	char *argData;

	index = getArgPointer(argc, argv, index, &argData, parmDesc);

	if (index < argc)
	{
		/* value is included with parameter */
		captureString(parm, argData, parmsize);
	}

	return index;
}

void processArgs(int argc, char **argv)

{
	int		i;
	int		foundit;
	char	ch;

	i = 1;
	while ((i < argc) && (0 == err))
	{
		foundit = 0;

		if ('-' == argv[i][0])
		{
			ch = toupper(argv[i][1]);

			/* check for option */
			if ((0 == foundit) && ('Q' == ch))
			{
				foundit = 1;

				i = processIndArg(argc, 
								  argv, 
								  i,
								  saveQname, 
								  sizeof(saveQname), 
								  "queue name (-q)");
			}

			/* check for option */
			if ((0 == foundit) && ('M' == ch))
			{
				foundit = 1;

				i = processIndArg(argc, 
								  argv, 
								  i,
								  saveQMname, 
								  sizeof(saveQMname), 
								  "queue manager name (-m)");
			}

			/* check for option */
			if ((0 == foundit) && ('F' == ch))
			{
				foundit = 1;

				i = processIndArg(argc, 
								  argv, 
								  i,
								  parmFilename, 
								  sizeof(parmFilename), 
								  "name of parameters file (-f)");
			}

			/* check for option */
			if ((0 == foundit) && ('O' == ch))
			{
				foundit = 1;

				i = processIndArg(argc, 
								  argv, 
								  i,
								  outputFilename, 
								  sizeof(outputFilename), 
								  "name of output data file (-f)");
			}
		}

		/* did we recognize the parameter? */
		if (0 == foundit)
		{
			/* issue an error message */
			printf("*****Unrecognized command line parameter found %c\n", argv[i][0]);

			/* Set the return code based on the message number */
			err = 93;
		}

		i++;
	}
}

/**************************************************************/
/*                                                            */
/* Process a line in the parameters file.                     */
/*                                                            */
/**************************************************************/

int evaluateParm(char * ptr, char * value)

{
	int foundit=0;
	int len;
	char *valueptr;

	/* remove any quotes from the value */
	valueptr = removeQuotes(value);
	len = strlen(valueptr);

	if (strcmp(ptr, QNAME) == 0)
	{
		foundit = 1;
		if (len > MQ_Q_NAME_LENGTH)
		{
			/* parameter too long, issue warning and truncate */
			printf("***** Queue name - parameter too long (%d) - data truncated - (%s)\n", 
					len, value);

			len = MQ_Q_NAME_LENGTH;
		}

		/* check for previous setting */
		if (qname[0] > ' ')
		{
			/* issue warning message */
			printf("***** output queue name has been specified previously, new value (%d) used\n",
					valueptr);
		}

		/* get the output queue name */
		memset(qname, 0, sizeof(qname));
		memcpy(qname, valueptr, len);
	}

	if (strcmp(ptr, QMGR) == 0)
	{
		foundit = 1;
		if (len > MQ_Q_MGR_NAME_LENGTH)
		{
			/* parameter too long, issue warning and truncate */
			printf("***** Queue manager name - parameter too long (%d) - data truncated - (%s)\n", 
					len, value);

			len = MQ_Q_MGR_NAME_LENGTH;
		}

		/* check for previous setting */
		if (qmname[0] > ' ')
		{
			/* issue warning message */
			printf("***** output queue manager has been specified previously, new value (%d) used\n",
					valueptr);
		}

		/* get the output queue manager name to connect to */
		memset(qmname, 0, sizeof(qmname));
		memcpy(qmname, valueptr, len);
	}

	if (strcmp(ptr, DELIMITER) == 0)
	{
		foundit = 1;
	}

	if (strcmp(ptr, DELIMITERX) == 0)
	{
		foundit = 1;
	}

	if (strcmp(ptr, STRIPRFH) == 0)
	{
		foundit = 1;

		if (('y' == valueptr[0]) || ('Y' == valueptr[0]))
		{
			striprfh = RFHSTRIP;
		}
		else
		{
			striprfh = RFHNO;
		}
	}

	if (strcmp(ptr, MAXMSGCOUNT) == 0)
	{
		foundit = 1;
	}

	return foundit;
}

/**************************************************************/
/*                                                            */
/* Find the parameter and the value.                          */
/*                                                            */
/**************************************************************/

void processParmLine(char * ptr)

{
	int		foundit=0;
	char *	valueptr;
	char *	blankptr;

	/* find the equal equal sign */
	valueptr = strchr(ptr, '=');

	/* did we find one? */
	if (valueptr != NULL)
	{
		/* break the line into name and value parts */
		valueptr[0] = 0;
		valueptr++;

		/* find the beginning of the value */
		valueptr = skipBlanks(valueptr);
	}

	/* strip trailing blanks and translate the name to upper case */
	blankptr = findBlank(ptr);
	blankptr[0] = 0;
	strupper(ptr);

	/* is this a name value pair? */
	if (valueptr != NULL)
	{
		/* check that we have a value */
		if (valueptr[0] > 0)
		{
			foundit = evaluateParm(ptr, valueptr);

			if (foundit == 0)
			{
				printf("***** unrecognized parameter %s, value %s\n", ptr, valueptr);
			}
		}
		else
		{
			printf("***** missing or blank value for parameter %s\n", ptr);
		}
	}
}

void processParms(FILE *parmfile)

{
	int		len;
	char *	ptr;
	char	parmline[512];

	/* read the parameter file */
	while (fgets(parmline, sizeof(parmline) - 1, parmfile) != NULL)
	{
		/* check for a new line character at the end of the line */
		while ((strlen(parmline) > 0) && (parmline[strlen(parmline) - 1] < ' '))
		{
			/* get rid of the new line character */
			parmline[strlen(parmline) - 1] = 0;
		}

		/* point to the beginning of the line */
		/* skip any leading blanks */
		ptr = skipBlanks(parmline);

		/* truncate any trailing blanks */
		len = strlen(ptr);
		while ((len > 0) && (ptr[len] == ' '))
		{
			ptr[len] = 0;
			len--;
		}

		/* check for a comment or blank line */
		if ((ptr[0] != 0) && (ptr[0] != ';') && (ptr[0] != '#') && (ptr[0] != '*'))
		{
			/* process this line */
			processParmLine(ptr);

			/* re-initialize the parameter input area */
			memset(parmline, 0, sizeof(parmline));
		}
	}
}

int processParmFile(char * filename)

{
	int		rc=0;
	int		len=0;
	FILE	*parmfile;

	parmfile = fopen(filename, "r");

	if (NULL == parmfile)
	{
		rc = 1;
		printf("unable to open parameters file %s\n", filename);
	}
	else
	{
		/* process the parameters file */
		processParms(parmfile);

		/* close the parameters file */
		fclose(parmfile);
	}

	return rc;
}

int checkRFH(const char * msgdata, const int datalen)

{
	MQRFH2	tempRFH;
	char	tempEbcdic[32];
	int		rfhlength=0;
	int		rfhversion;
	int		ebcdic=0;

	/* check if the message is long enough to have an RFH */
	if (datalen < sizeof(tempRFH))
	{
		return 0;
	}

	/* get a copy of the first 36 bytes of the message */
	memcpy(&tempRFH, msgdata, sizeof(tempRFH));

	if (memcmp(tempRFH.StrucId, MQRFH_STRUC_ID, sizeof(tempRFH.StrucId)) != 0)
	{
		/* try converting the StrucId field to EBCDIC */
		EbcdicToAscii((unsigned char *) &tempRFH.StrucId, sizeof(tempRFH.StrucId), (unsigned char *)tempEbcdic);
		memcpy(&tempRFH.StrucId, tempEbcdic, sizeof(tempRFH.StrucId));
		ebcdic = 1;

		/* see if we have a structure identifier in EBCDIC */
		if (memcmp(tempRFH.StrucId, MQRFH_STRUC_ID, sizeof(tempRFH.StrucId)) != 0)
		{
			/* no RFH found */
			return 0;
		}
	}

	/* see if we can recognize the version and length */
	rfhversion = tempRFH.Version;
	rfhlength = tempRFH.StrucLength;

	/* check if we have the right encoding */
	if ((MQRFH_VERSION_1 != rfhversion) && (MQRFH_VERSION_2 != rfhversion))
	{
		/* reverse the bytes and see if we recognize the version */
		rfhversion = reverseBytes4(rfhversion);
		rfhlength = reverseBytes4(rfhlength);

		if ((MQRFH_VERSION_1 != rfhversion) && (MQRFH_VERSION_2 != rfhversion))
		{
			return 0;
		}
	}

	/* finally check that the length makes sense */
	if ((rfhlength < MQRFH_STRUC_LENGTH_FIXED) || (rfhlength > datalen))
	{
		/* length doesn't make sense */
		return 0;
	}

	return rfhlength;
}

void printHelp(char *pgmName)

{
	printf("%s\n", Level);
	printf("format is:\n");
	printf("   %s -f parm-filename -o data_filename \n", pgmName);
}

int main(int argc, char **argv)

{
	int		rc;
	int		msgcount=0;
	int		maxrate=0;
	int		firstsec=0;
	int		secondcount=0;
	int		totalbytes=0;
	int		firstTime=0;
	int		rfhlength=0;
	MQLONG	qm=0;
	MQLONG	q=0;
	MQLONG	compcode;
	MQLONG	reason;
	MQOD	objdesc = {MQOD_DEFAULT};
	MQMD	msgdesc = {MQMD_DEFAULT};
	MQLONG	openopt = 0;
	MQGMO	mqgmo = {MQGMO_DEFAULT};
	MQLONG	datalen=0;
	FILE	*outFile;
	char	*msgdata;
	char	*dataptr;

	printf("MQCAPONE V1.1 program start\n");

	/* print the copyright statement */
	printf("Copyright (c) IBM Corp., 2001/2002\n");

	/* allocate a buffer for the message */
	msgdata = (char *)malloc(MAX_MESSAGE_LENGTH);

	/* initialize the work areas */
	memset(parmFilename, 0, sizeof(parmFilename));
	memset(outputFilename, 0, sizeof(outputFilename));
	memset(qmname, 0, sizeof(qmname));
	memset(qname, 0, sizeof(qname));
	memset(saveQMname, 0, sizeof(saveQMname));
	memset(saveQname, 0, sizeof(saveQname));
	memset(msgdata, 0, sizeof(msgdata));

	/* check for too few input parameters */
	if (argc < 3)
	{
		printHelp(argv[0]);
		exit(99);
	}

	/* check for help request */
	if ((argv[1][0] == '?') || (argv[1][1] == '?'))
	{
		printHelp(argv[0]);
		exit(0);
	}

	/* process any command line arguments */
	processArgs(argc, argv);

	if (err != 0)
	{
		printHelp(argv[0]);
		exit(99);
	}

	if ((0 == parmFilename[0]) || (0 == outputFilename[0]))
	{
		printHelp(argv[0]);
		exit(99);
	}

	/* tell what parameters file we are using */
	printf("Reading parameters from file %s\n", parmFilename);

	/* read the ini file to get the delimiter string */
	rc = processParmFile(parmFilename);

	/* return if unable to find parameters file */
	if (rc != 0)
	{
		return rc;
	}

	/* check for overrides */
	if (saveQMname[0] != 0)
	{
		strcpy(qmname, saveQMname);
	}

	/* check for overrides */
	if (saveQname[0] != 0)
	{
		strcpy(qname, saveQname);
	}

	/* exit if queue name not found */
	if (0 == qname[0])
	{
		printf("***** Queue name not found in parameters file\n");
		return 2;
	}

	/* Tell what queue and qm will be used */
	if (0 == qmname[0])
	{
		printf("Reading messages from Queue(%s) on default Qmgr\n", qname);
	}
	else
	{
		printf("Reading messages from Queue(%s) on Qmgr(%s)\n", qname, qmname);
	}

	/* open output file */
	printf("opening output file %s\n", outputFilename);
	outFile = fopen(outputFilename, "wb");

	if (NULL == outFile)
	{
		printf("unable to open output file %s for output\n", outputFilename);
		return 100;
	}

	/* Connect to the queue manager */
	printf("connecting to queue manager %s\n",qmname);
	MQCONN(qmname, &qm, &compcode, &reason);

	/* check for errors */
	checkerror("MQCONN", compcode, reason, qmname);
	if (compcode != MQCC_OK)
	{
		return 98;
	}

	/* set the queue open options */
	strncpy(objdesc.ObjectName, qname, MQ_Q_NAME_LENGTH);
	openopt = MQOO_INPUT_SHARED + MQOO_FAIL_IF_QUIESCING;

	/* open the queue for input */
	printf("opening queue %s for input\n", qname);
	MQOPEN(qm, &objdesc, openopt, &q, &compcode, &reason);

	/* check for errors */
	checkerror("MQOPEN", compcode, reason, qname);
	if (compcode != MQCC_OK)
	{
		return 97;
	}

	/* set the get message options */
	mqgmo.Version = MQGMO_VERSION_2;
	mqgmo.Options = MQGMO_NO_WAIT | MQGMO_FAIL_IF_QUIESCING;
	mqgmo.MatchOptions = MQGMO_NONE;

	/* reset the msgid and correlid */
	memcpy(msgdesc.MsgId, MQMI_NONE, sizeof(msgdesc.MsgId));
	memcpy(msgdesc.CorrelId, MQCI_NONE, sizeof(msgdesc.CorrelId));

	/* perform the MQGET */
	MQGET(qm, q, &msgdesc, &mqgmo, MAX_MESSAGE_LENGTH, msgdata, &datalen, &compcode, &reason);

	/* check for errors */
	checkerror("MQGET", compcode, reason, qname);

	if (compcode == MQCC_OK)
	{
		/* count the total number of messages read */
		msgcount++;

		/* point to the data area */
		dataptr = msgdata;

		/* check for an MQMDE present */
		if ((datalen >= MQMDE_LENGTH_2) && (memcmp(dataptr, MQMDE_STRUC_ID, sizeof(MQMDE_STRUC_ID) - 1) == 0))
		{
			dataptr += MQMDE_LENGTH_2;
			datalen -= MQMDE_LENGTH_2;
			printf("MQMD extension removed from data\n");
		}

		/* do we need to check for an RFH at the front of the data? */
		if (RFHSTRIP == striprfh)
		{
			/* check the message format for an RFH */
			if ((memcmp(msgdesc.Format, MQFMT_RF_HEADER, sizeof(MQFMT_RF_HEADER) - 1) == 0) ||
				(memcmp(msgdesc.Format, MQFMT_RF_HEADER_2, sizeof(MQFMT_RF_HEADER_2) - 1) == 0))
			{
				rfhlength = checkRFH(dataptr, datalen);
			}
		}

		/* calculate the total bytes in the message */
		totalbytes += datalen;

		/* append the data to the file */
		fwrite(dataptr + rfhlength, datalen - rfhlength, 1, outFile);
	}

	/* close the file */
	fclose(outFile);

	/* close the input queue */
	printf("closing the input queue\n");
	MQCLOSE(qm, &q, MQCO_NONE, &compcode, &reason);

	checkerror("MQCLOSE", compcode, reason, qname);

	/* Disconnect from the queue manager */
	printf("disconnecting from the queue manager\n");
	MQDISC(&qm, &compcode, &reason);

	checkerror("MQDISC", compcode, reason, qmname);

	/* dump out the total message count */
	printf("Total messages %d\n", msgcount);

	/* give the total and average message size */
	if (msgcount > 0)
	{
		printf("total bytes in message %d\n", totalbytes);
	}

	/******************************************************************/
	/*                                                                */
	/* END OF PROGRAM                                                 */
	/*                                                                */
	/******************************************************************/

	printf("mqcapone program ended\n");
	return 0;
}
